@@ -4,62 +4,55 @@ package Apache2::ASP::TransHandler;
 use strict;
 use APR::Table ();
 use Apache2::RequestRec ();
-use Apache2::RequestUtil ();
-use Apache2::SubRequest ();
 use Apache2::Const -compile => ':common';
-use Apache2::ServerRec ();
-
-use Apache2::ASP::GlobalConfig;
-
-our %configs = ();
 
 sub handler : method
 {
   my ($class, $r) = @_;
   
-  $class->init_config( $r );
+  # Get our config:
+  $ENV{APACHE2_ASP_CONFIG} = $ENV{APACHE2_ASP_GLOBALCONFIG}->find_current_config();
   
   # Fixup the request URI:
   my $filename = $r->filename;
-
   if( -d $filename )
   {
     # See if there is an index.asp here:
     if( -f $filename . "index.asp" )
     {
       $r->filename( $filename . "index.asp" );
-      $r->uri( $r->uri . "/index.asp" );
-      return -1;
+      return $class->handler( $r );
     }
     else
     {
       # We don't do directory indexes:
       return 403;
     }# end if()
+  }
+  else
+  {
+    # Fixup /media/* URL requests:
+    return Apache2::Const::DECLINED()
+      unless $r->uri =~ m/^\/media\/.+/;
+    my ($file) = $r->uri =~ m/^\/media\/([^\?]+)$/;
+    
+    my @args = ( "file=$file" );
+    if( $r->args )
+    {
+      push @args, $r->args;
+    }# end if()
+    
+    # Fixup the uri and args:
+    $r->uri( '/handlers/MediaManager' );
+    $r->args( join '&', @args );
+    
+    # Send the request on down the line to the next handler:
+    return Apache2::Const::DECLINED();
   }# end if()
   
   return -1;
 }# end handler()
 
-
-sub init_config
-{
-  my ($s, $r) = @_;
-  
-	my $domain = $ENV{HTTP_HOST} = $r->hostname || $r->server->server_hostname;
-	if( $configs{$domain} )
-	{
-		$ENV{APACHE2_ASP_GLOBAL_CONFIG} = $configs{$domain};
-	}
-	else
-	{
-		$ENV{APACHE2_ASP_GLOBAL_CONFIG} = $configs{$domain} = Apache2::ASP::GlobalConfig->new();
-	  warn "Apache2::ASP::GlobalConfig($domain) has been loaded into \$ENV{APACHE2_ASP_GLOBALCONFIG}\n";
-	}# end if()
-	
-	$ENV{APACHE2_ASP_CONFIG} = $ENV{APACHE2_ASP_GLOBAL_CONFIG}->find_current_config( $r );
-}# end init_config()
-
 1;# return true:
 
 __END__
@@ -90,7 +83,7 @@ Use RT L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=Apache2-ASP> to submit bug rep
 
 =head1 HOMEPAGE
 
-Please visit the Apache2::ASP homepage at L<http://www.devstack.com/> to see examples
+Please visit the Apache2::ASP homepage at L<http://apache2-asp.no-ip.org/> to see examples
 of Apache2::ASP in action.
 
 =head1 AUTHOR
